-- tòmò Web Analytics
-- Copyright since 2011 by Yves Goergen
-- Website: http://unclassified.de/go/tomo
-- Licence: GNU General Public License (GPL) Version 3
-- Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
--
-- This file contains the database schema SQL script for SQLite.

-- ------------------------------------------------------------------------------------------------
-- ATTENTION:
-- When adding new columns referencing to the text table, make sure to also include them in the
-- PurgeText method in the code, to not accidentially delete text entries still in use.
-- ------------------------------------------------------------------------------------------------

PRAGMA foreign_keys = ON;

-- Create base relations

CREATE TABLE "setting" (
	"Name" VARCHAR NOT NULL,
	"Value" VARCHAR,
	PRIMARY KEY ("Name"));

CREATE TABLE "text" (
	"TextId" INTEGER NOT NULL,
	"Value" VARCHAR NOT NULL,
	PRIMARY KEY ("TextId"));
CREATE INDEX "text_Value_idx" ON "text" ("Value");

CREATE TABLE "visitor" (
	"VisitorId" INTEGER NOT NULL PRIMARY KEY);

CREATE TABLE "session" (
	"SessionId" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	"VisitorId" INTEGER REFERENCES "visitor" ("VisitorId"),
	"NewVisitor" BOOLEAN NOT NULL,
	-- equal to ActionTime of first action in the session (for better performance)
	"SessionStartTime" DATETIME NOT NULL,
	-- currently of unknown use
	"SessionDuration" INTEGER,
	"ReferrerSsl" BOOLEAN,
	"ReferrerHostText" INTEGER REFERENCES "text" ("TextId"),
	"ReferrerUrlPathText" INTEGER REFERENCES "text" ("TextId"),
	"SearchEngineText" INTEGER REFERENCES "text" ("TextId"),
	"SearchKeywordsText" INTEGER REFERENCES "text" ("TextId"),
	"SearchImageUrlText" INTEGER REFERENCES "text" ("TextId"),
	"SearchResultPage" INTEGER,
	"CampaignText" INTEGER REFERENCES "text" ("TextId"),
	"IPAddress" VARCHAR NOT NULL,
	"IPVersion" INTEGER NOT NULL,
	"ProviderText" INTEGER REFERENCES "text" ("TextId"),
	"DNT" BOOLEAN,
	"CountryCode" CHAR(2),
	"RegionText" INTEGER REFERENCES "text" ("TextId"),
	"CityText" INTEGER REFERENCES "text" ("TextId"),
	"Lat" INTEGER,
	"Lng" INTEGER,
	"UtcOffset" INTEGER,
	"BrowserText" INTEGER REFERENCES "text" ("TextId"),
	"OperatingSystemText" INTEGER REFERENCES "text" ("TextId"),
	"Mobile" BOOLEAN,
	"Language" VARCHAR,
	"JavaScript" BOOLEAN,
	"Cookies" BOOLEAN,
	"Flash" BOOLEAN,
	"Java" BOOLEAN,
	"Quicktime" BOOLEAN,
	"Silverlight" BOOLEAN,
	"WindowsMedia" BOOLEAN,
	"AdBlock" BOOLEAN,
	"ScreenWidth" INTEGER,
	"ScreenHeight" INTEGER,
	"ScreenAvailWidth" INTEGER,
	"ScreenAvailHeight" INTEGER,
	"ScreenColors" INTEGER,
	"ViewportWidth" INTEGER,
	"ViewportHeight" INTEGER,
	"SessionUpdateTime" DATETIME,
	-- temporary storage for full strings, used for session similarity detection when no cookies are supported
	-- TODO: make sure that these values are deleted for expired sessions
	"UserAgentString" VARCHAR,
	"LanguagesString" VARCHAR);
CREATE INDEX "session_SessionStartTime_idx" ON "session" ("SessionStartTime");
CREATE INDEX "session_VisitorId_idx" ON "session" ("VisitorId");

CREATE TABLE "pageview" (
	"PageviewId" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	"SessionId" INTEGER NOT NULL REFERENCES "session" ("SessionId"),
	"PageviewTime" DATETIME NOT NULL,
	"DomainText" INTEGER REFERENCES "text" ("TextId"),
	"UrlPathText" INTEGER NOT NULL REFERENCES "text" ("TextId"),
	"UrlQueryText" INTEGER REFERENCES "text" ("TextId"),
	"UrlHashText" INTEGER REFERENCES "text" ("TextId"),
	"Ssl" BOOLEAN NOT NULL,
	"PageTitleText" INTEGER REFERENCES "text" ("TextId"),
	"PageLanguage" VARCHAR,
	"ActivityTimespan" INTEGER,
	"PageLoadTime" INTEGER,
	"EntryPage" BOOLEAN NOT NULL,
	"ExitPage" BOOLEAN,
	"JavaScriptData" BOOLEAN NOT NULL,
	"CampaignRedirect" BOOLEAN NOT NULL,
	"Prerender" BOOLEAN NOT NULL,
	"OriginalPageviewId" INTEGER,
	"PageviewUpdateTime" DATETIME);
CREATE INDEX "sessvar_PageviewTime_idx" ON "pageview" ("PageviewTime");
CREATE INDEX "sessvar_SessionId_idx" ON "pageview" ("SessionId");
CREATE INDEX "sessvar_sid_time_idx" ON "pageview" ("SessionId", "PageviewTime");

CREATE TABLE "action" (
	"ActionId" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
	"PageviewId" INTEGER NOT NULL REFERENCES "pageview" ("PageviewId"),
	"ActionTime" DATETIME NOT NULL,
	"ActionType" INT NOT NULL,
	"ActionUrlText" INTEGER REFERENCES "text" ("TextId"),
	"ActionDataText" INTEGER REFERENCES "text" ("TextId"));
CREATE INDEX "action_ActionTime_idx" ON "action" ("ActionTime");
CREATE INDEX "action_PageviewId_idx" ON "action" ("PageviewId");
CREATE INDEX "action_pid_time_idx" ON "action" ("PageviewId", "ActionTime");

CREATE TABLE "lock" (
	"LockId" VARCHAR NOT NULL PRIMARY KEY,
	"LockCreateTime" DATETIME NOT NULL);

